/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is from autoform.
 *
 * The Initial Developer of the Original Code is
 * Michael Kessler, Bluewin AG <michael.kessler@team.bluewin.com>.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Balabanov Dmitry <Balabanov@iso.ru>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
var autoPrefs = prefService.getBranch("autoform.");
var rootPrefs = prefService.getBranch(null);

var sbSrv = Components.classes['@mozilla.org/intl/stringbundle;1'].createInstance();
var sbSrvI = sbSrv.QueryInterface(Components.interfaces.nsIStringBundleService);

var autoformBundle = sbSrvI.createBundle("chrome://autoform/locale/autoform.properties");

var virtualUserPrefs,autouser;
var userChanged = true;

var masterpassword = null;
   
var prefCount = {
   value : 0
};
   
//************************************************/
// read boolean preference value, return default value if not found
//************************************************/

function getBoolPref(name, defaultvalue) {
   try {
			   if(virtualUserPrefs.prefHasUserValue(name)) {
			      return virtualUserPrefs.getBoolPref(name);
			   }
			
			   virtualUserPrefs.setBoolPref(name, defaultvalue);
			
			   return defaultvalue;

   } catch(e) {
      debug(e, "getBoolPref()");
   } 
}

//**********************************************************/
// read virtual user character preference value, return default value if not found
//**********************************************************/

function getCharPref(name, defaultvalue) {
   try {
			   if(virtualUserPrefs.prefHasUserValue(name)) {
			      return virtualUserPrefs.getCharPref(name);
			   }
			
			   virtualUserPrefs.setCharPref(name, defaultvalue);
			
			   return defaultvalue;

   } catch(e) {
      debug(e, "getCharPref()");
   }    
}

//*****************************************************/
// read main character preference value, return default value if not found
//*****************************************************/

function getCharPrefFromMain(name, defaultvalue) {
   try {
			   if(autoPrefs.prefHasUserValue(name)) {
			      return autoPrefs.getCharPref(name);
			   }
			
			   autoPrefs.setCharPref(name, defaultvalue);
			
			   return defaultvalue;

   } catch(e) {
      debug(e, "getCharPrefFromMain()");
   }    
}

//************************************************/
// read preference value of unknown type
//************************************************/

function getUnknownPref(name) {
   try {
			   if(virtualUserPrefs.prefHasUserValue(name)) {
			
			      prefType = virtualUserPrefs.getPrefType(name);
			
			      if(prefType == 32) {
			         return virtualUserPrefs.getCharPref(name);
			      }
			      else if(prefType == 64) {
			         return virtualUserPrefs.getIntPref(name);
			      }
			      else if(prefType == 128) {
			         return virtualUserPrefs.getBoolPref(name);
			      }
			   }
			
			   return "";
   
   } catch(e) {
      debug(e, "getUnknownPref()");
   }    
}

//************************************************/
// read preference value of unknown type from main
//************************************************/

function getUnknownPrefFromBranch(branchname, name) {
   try {
						var branch = prefService.getBranch(branchname);
			   
			   if(branch.prefHasUserValue(name)) {
			
			      prefType = branch.getPrefType(name);
			
			      if(prefType == 32) {
			         return branch.getCharPref(name);
			      }
			      else if(prefType == 64) {
			         return branch.getIntPref(name);
			      }
			      else if(prefType == 128) {
			         return branch.getBoolPref(name);
			      }
			   }
			
			   return "";
			   
   } catch(e) {
      debug(e, "getUnknownPrefFromBranch()");
   } 	   
}

//************************************************/
// read preference type
//************************************************/

function getPrefTypeFromBranch(branchname, name) {
   try {
						var branch = prefService.getBranch(branchname);
			   
			   if(branch.prefHasUserValue(name)) {
			
			      prefType = branch.getPrefType(name);
			
			      if(prefType == 32) {
			         return "char";
			      }
			      else if(prefType == 64) {
			         return "int";
			      }
			      else if(prefType == 128) {
			         return "bool";
			      }
			   }
			
			   return "";
			   
   } catch(e) {
      debug(e, "getPrefTypeFromBranch()");
   } 	   
}

//************************************************/
// initialize preferences
//************************************************/

function doInitPreferences() {
	  try {
			   this.autouser = getCharPrefFromMain("autouser", "user0");
			   doInitializeUsers("autoform-users");
   } catch(e) {
      debug(e, "doInitPreferences()");
   } 	   
}

//************************************************/
// load preferences
//************************************************/

function doLoadPreferences() {
   try {
			   // autoform user
			   this.autouser = getCharPrefFromMain("autouser", "user0");
			   virtualUserPrefs = prefService.getBranch("autoform." + this.autouser + ".");
			
			   // alert
			   document.getElementById("autoform-erroralert").checked = getBoolPref("erroralert", false);

			   // Automation
			   document.getElementById("autoform-autoload").checked = getBoolPref("autoload", false);
			   document.getElementById("autoform-autosave").checked = getBoolPref("autosave", false);
			   document.getElementById("autoform-mozillaformfill").checked = getBoolPref("mozillaformfill", true);
			
			   // Action
			   document.getElementById("autoform-visualize").checked = getBoolPref("visualize", true);
			   document.getElementById("autoform-overwrite").checked = getBoolPref("overwrite", true);
			   document.getElementById("autoform-emptyvalues").checked = getBoolPref("emptyvalue", false);
			
			   // Password
			   document.getElementById("autoform-savepasswords").checked = getBoolPref("savepasswords", true);
			   document.getElementById("autoform-encryptpasswords").checked = getBoolPref("encryptpasswords", true);
			   document.getElementById("autoform-mozillapasswords").checked = getBoolPref("mozillapasswords", true);
			
			   // Field hash
			   document.getElementById("autoform-hash-hostname").checked = getBoolPref("hashhostname", true);
			   document.getElementById("autoform-hash-pathname").checked = getBoolPref("hashpathname", true);
			   document.getElementById("autoform-hash-search").checked = getBoolPref("hashsearch", true);
			   document.getElementById("autoform-hash-form").checked = getBoolPref("hashform", true);
			
			   // Toolbar buttons
			   document.getElementById("autoform-load-form").checked = getBoolPref("barloadform", true);
			   document.getElementById("autoform-save-form").checked = getBoolPref("barsaveform", true);
			   document.getElementById("autoform-reset-form").checked = getBoolPref("barresetform", true);
			   document.getElementById("autoform-delete-form").checked = getBoolPref("bardeleteform", true);
			   document.getElementById("autoform-config").checked = getBoolPref("barconfig", true);
			   document.getElementById("autoform-help").checked = getBoolPref("barhelp", true);
			   document.getElementById("autoform-user-select").checked = getBoolPref("baruserselect", false);
			
			   // Button style
			   document.getElementById("autoform-button-style").selectedItem = document.getElementById(getCharPref("barstyle", "icontext"));
			
			   // Load Lists
			   doInitializeList("autoform-loadNever");
			   doInitializeList("autoform-loadAlways");
			   doInitializeList("autoform-saveNever");
			   doInitializeList("autoform-saveAlways");
			   doInitializeList("autoform-excludeParameters");
			
			   // do regexp check
			   doCheckRegexp('loadAlwaysTestBox', 'autoform-loadAlways', 'statusAlwaysLoad');
			   doCheckRegexp('loadNeverTestBox', 'autoform-loadNever', 'statusNeverLoad');
			   doCheckRegexp('saveNeverTestBox', 'autoform-saveNever', 'statusNeverSave');
			   doCheckRegexp('saveAlwaysTestBox', 'autoform-saveAlways', 'statusAlwaysSave');
			   
			   selTab = document.getElementById("autoform-tabbox").selectedTab;
			   
						if (selTab.id == "autoform-tab-editvalues") {
							  doPreparePreferenceEdit();
						   userChanged = false;
						} else {
						   userChanged=true;
						}
			
   } catch(e) {
      debug(e, "doLoadPreferences()");
   } 			
}

//************************************************/
// fill preference edit containers
//************************************************/

function doPreparePreferenceEdit() {
			try {	
				this.autouser = getCharPrefFromMain("autouser", "user0");
						
				var dataPrefs = prefService.getBranch("autoform." + this.autouser + ".formdata.");
			   var prefArray = dataPrefs.getChildList("", prefCount);
			   var topParent = document.getElementById("autoform-pref-edit-children");
			   
			   var meter = document.getElementById("autoform-pref-progress");
			   meter.setAttribute('mode', 'determined');
			   
			   var maxsteps = (5*prefCount.value) + topParent.childNodes.length;
			   var steps = 0;
			   var progress = 0;
			   
			   // remove old data
			   while(topParent.firstChild != null) {
			      topParent.removeChild(topParent.firstChild);
			      steps += 1;
			      progress=Math.round((100/maxsteps)*steps);
			      meter.setAttribute('value', progress + '%');
			   }
			         
			   // fill data
						for (l=0; l<5; l++) {			
			   			for(p=0; p<prefCount.value; p++) {
			   				
											var prefTokens = new tokenizer(prefArray[p], ".");
												
											// id from actual element
										  var tokenId = "";
										  for (i=0; i<l+1;i++) {
										  			tokenId += prefTokens[i] + ".";
										  }
										  
										  // id from actual element's parent
										  var tokenIdParent = "";
										  for (i=0; i<l;i++) {
										  			tokenIdParent += prefTokens[i] + ".";
										  }
										  							  
												if (prefTokens.length == l) {
															// last token --> item
				           var treeItem = document.createElement("treeitem");
				           treeItem.setAttribute("id", tokenId);
				           treeItem.setAttribute("branch",tokenIdParent);
				           
				           var treeRow = document.createElement("treerow");
				           
				           // add description
				           var descType = new tokenizer(doHexDecode(prefTokens[l]), "@@@");
				           var treeCellDesc = document.createElement("treecell");
				           treeCellDesc.setAttribute("label", descType[0]);
				           treeCellDesc.setAttribute("value", prefArray[p]);
				           treeCellDesc.setAttribute("id", "DESC" + tokenId);
			
				           // add type
				           var treeCellType = document.createElement("treecell");
				           treeCellType.setAttribute("label", descType[1]);
				           treeCellType.setAttribute("id", "TYPE" + tokenId);
			
															// add value
				           var treeCellValue = document.createElement("treecell");
				           treeCellValue.setAttribute("id", "VALUE" + tokenId);
															treeCellValue.setAttribute("elementId", prefTokens[l]);	 
															          
															var theValue = doHexDecode(getUnknownPrefFromBranch("autoform." + this.autouser + ".formdata." + tokenIdParent, prefTokens[l]));
															
				           if (descType[1] == "select-one" || descType[1] == "select-multiple") {
				  			         
																		var descTokens = new tokenizer(theValue, "|@|");
																		var desc = "";
																		
																		for (t=0; t<descTokens.length; t++) {
																		   desc += descTokens[t];
																		   if (t != descTokens.length -1) {
																		   	  desc += ",";
																		   }
																		}
																		
							           treeCellValue.setAttribute("label", desc);
							           treeCellValue.setAttribute("value", theValue);
				           } else {
																		if (descType[1] != 'password') {
										           treeCellValue.setAttribute("label", theValue);
																		} else {
										           treeCellValue.setAttribute("label", "********");
																		}
			
				  			         treeCellValue.setAttribute("value", theValue);
				           }
															
				           treeRow.appendChild(treeCellDesc);
				           treeRow.appendChild(treeCellType);
				           treeRow.appendChild(treeCellValue);
				           treeItem.appendChild(treeRow);
				           parent.appendChild(treeItem);
			
				           if (l!=0) {
				           	  var parent = document.getElementById(tokenIdParent);
							           parent.appendChild(treeItem);
				           } else {
							           topParent.appendChild(treeItem);
				           }
				           
												} else {
						
													  if (prefTokens.length > l && document.getElementById(tokenId) == null) {
																  // not last token --> container
							           var treeItem = document.createElement("treeitem");
							           treeItem.setAttribute("container", true);
							           var treeRow = document.createElement("treerow");
							           var treeCell = document.createElement("treecell");
							           var treeChildren = document.createElement("treechildren");
							           treeChildren.setAttribute("id",tokenId);
							           treeCell.setAttribute("label", doHexDecode(prefTokens[l]));
							           treeCell.setAttribute("value", prefArray[p]);
							           treeRow.appendChild(treeCell);
							           treeItem.appendChild(treeRow);
							           treeItem.appendChild(treeChildren);
			
							           if (l!=0) {
							           	  var parent = document.getElementById(tokenIdParent);
										           parent.appendChild(treeItem);
							           } else {
							              topParent.appendChild(treeItem);
							           }
													  }
												}
			
			 		      steps += 1;
			 		      progress=Math.round((100/maxsteps)*steps);
						      meter.setAttribute('value', progress + '%');
												
			   			}
						}

   } catch(e) {
      debug(e, "doPreparePreferenceEdit()");
   } 			
} 

//************************************************/
// check if user switch to edit data tab
//************************************************/

function doTabChange() {
	  try {
			   selTab = document.getElementById("autoform-tabbox").selectedTab;
			   
						if (selTab.id == "autoform-tab-editvalues" && userChanged) {
												var meter = document.getElementById("autoform-pref-progress");
												meter.setAttribute("value", "0%");
												
												// give some time to switch tab, otherwise we don't see
												// the progress bar
							  			setTimeout("doPreparePreferenceEdit()", 100);
							  			userChanged = false;
						}
   } catch(e) {
      debug(e, "doTabChange()");
   } 
}

//************************************************/
// init user list
//************************************************/

function doUserList(name) {
   try {
			   // clear first
			   var list = document.getElementById(name);
			
			   while(list.getRowCount() != 0) {
			      list.removeItemAt(0);
			   }
			
			   // initialize
			   var listPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("autoform." + name + ".");
			
			   var prefCount = {
			      value : 0
			   };
			
			   var prefArray = listPrefs.getChildList("", prefCount);
			
			   for(i = 0; i < prefCount.value; i++) {
			      list.appendItem(listPrefs.getCharPref(prefArray[i]), prefArray[i]);
			   }
   
   } catch(e) {
      debug(e, "doUserList()");
   }     
}

//************************************************/
// fill list with values
//************************************************/

function doInitializeList(name) {
   try {
			   // clear first
			   var list = document.getElementById(name);
			
			   while(list.getRowCount() != 0) {
			      list.removeItemAt(0);
			   }
			
			   // initialize
			   var listPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("autoform." + this.autouser + "." + name + ".");
			   var prefCount = {
			      value : 0
			   };
			   var prefArray = listPrefs.getChildList("", prefCount);
			
			   for(i = 0; i < prefCount.value; i++) {
			      list.appendItem(doHexDecode(listPrefs.getCharPref(prefArray[i])), prefArray[i]);
			   }
   
   } catch(e) {
      debug(e, "doInitializeList()");
   }     
}

//************************************************/
// initialize users
//************************************************/

function doInitializeUsers() {
   try {
			   // clear first
			   var list = document.getElementById("autoform-users");
			   while(list.getRowCount() != 0) {
			      list.removeItemAt(0);
			   }
			
			   // initialize
			   var listPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("autoform.autoform-users.");
			   var prefCount = {
			      value : 0
			   };
			
			   var prefArray = listPrefs.getChildList("", prefCount);
			   var selItem;
			
			   for(i = 0; i < prefCount.value; i++) {
			      if(this.autouser == prefArray[i]) {
			         // add item
			         selItem = list.appendItem(doHexDecode(listPrefs.getCharPref(prefArray[i])), prefArray[i]);
			      }
			      else {
			         // add item
			         list.appendItem(doHexDecode(listPrefs.getCharPref(prefArray[i])), prefArray[i]);
			      }
			   }
			
						// invokes doLoadPreferences() throught onselect event
			   list.selectItem(selItem);
			   
   } catch(e) {
      debug(e, "doInitializeUsers()");
   }    
}

//************************************************/
// save preferences
//************************************************/

function doSavePreferences() {
   try {
			   // Alert
			   virtualUserPrefs.setBoolPref("erroralert", document.getElementById("autoform-erroralert").checked);

			   // Automation
			   virtualUserPrefs.setBoolPref("autoload", document.getElementById("autoform-autoload").checked);
			   virtualUserPrefs.setBoolPref("autosave", document.getElementById("autoform-autosave").checked);
			   virtualUserPrefs.setBoolPref("mozillaformfill", document.getElementById("autoform-mozillaformfill").checked);
			   
			   // Action
			   virtualUserPrefs.setBoolPref("visualize", document.getElementById("autoform-visualize").checked);
			   virtualUserPrefs.setBoolPref("overwrite", document.getElementById("autoform-overwrite").checked);
			   virtualUserPrefs.setBoolPref("emptyvalue", document.getElementById("autoform-emptyvalues").checked);
			   
			   // Password
			   virtualUserPrefs.setBoolPref("savepasswords", document.getElementById("autoform-savepasswords").checked);
			   virtualUserPrefs.setBoolPref("encryptpasswords", document.getElementById("autoform-encryptpasswords").checked);
			   virtualUserPrefs.setBoolPref("mozillapasswords", document.getElementById("autoform-mozillapasswords").checked);
			   
			   // Field hash
			   virtualUserPrefs.setBoolPref("hashhostname", document.getElementById("autoform-hash-hostname").checked);
			   virtualUserPrefs.setBoolPref("hashpathname", document.getElementById("autoform-hash-pathname").checked);
			   virtualUserPrefs.setBoolPref("hashsearch", document.getElementById("autoform-hash-search").checked);
			   virtualUserPrefs.setBoolPref("hashform", document.getElementById("autoform-hash-form").checked);
			   
			   // Toolbar buttons
			   virtualUserPrefs.setBoolPref("barloadform", document.getElementById("autoform-load-form").checked);
			   virtualUserPrefs.setBoolPref("barsaveform", document.getElementById("autoform-save-form").checked);
			   virtualUserPrefs.setBoolPref("barresetform", document.getElementById("autoform-reset-form").checked);
			   virtualUserPrefs.setBoolPref("bardeleteform", document.getElementById("autoform-delete-form").checked);
			   virtualUserPrefs.setBoolPref("barconfig", document.getElementById("autoform-config").checked);
			   virtualUserPrefs.setBoolPref("baruserselect", document.getElementById("autoform-user-select").checked);
			   virtualUserPrefs.setBoolPref("barhelp", document.getElementById("autoform-help").checked);
			   
			   // Button style
			   virtualUserPrefs.setCharPref("barstyle", document.getElementById("autoform-button-style").selectedItem.id);
			   
   } catch(e) {
      debug(e, "doSavePreferences()");
   } 
}

//************************************************/
// show plain text warning
//************************************************/

function doPlainTextWarning() {
	  try {
			   if(!document.getElementById("autoform-encryptpasswords").checked) {
			      alert(autoformBundle.GetStringFromName("plaintextwarning"));
			   }
			   
			   doSavePreferences();
   
   } catch(e) {
      debug(e, "doPlainTextWarning()");
   } 
}

//************************************************/
// remove selection from list
//************************************************/

function doRemoveSelectedItem(name) {
   try {
			   var list = document.getElementById(name);
			
			   for(i = 0; i < list.selectedCount; i++) {
			      if(name == 'autoform-users' && list.getSelectedItem(i).value == 'user0') {
			         alert(autoformBundle.GetStringFromName("removeuser") + " " + list.getSelectedItem(i).label + "!");
			      }
			      else {
			         list.removeItemAt(list.getIndexOfItem(list.getSelectedItem(i)));
			      }
			   }
			
			   if(name == "autoform-users") {
			      doSaveListToMain(name);
			      list.selectedIndex = 0;
			   }
			   else {
			      doSaveList(name);
			   }
   
   } catch(e) {
      debug(e, "doRemoveSelectedItem()");
   }      
}

//************************************************/
// add item to list
//************************************************/

function doAddItem(name, display) {
   try {
			   var value = window.prompt(autoformBundle.GetStringFromName("enterbefore") + " " + display + " " + autoformBundle.GetStringFromName("enterafter"), "");
			
			   if(display == 'regexp') {
			      try
			      {
			         myregexp = new RegExp(value);
			      }
			      catch(e) {
			         alert(autoformBundle.GetStringFromName("noregexp"));
			         return;
			      }
			   }
			
			   var list = document.getElementById(name);
			
			   if(value != null && value != "") {
			      var addedItem = list.appendItem(value, display + list.getRowCount());
			   }
			
			   if(name == "autoform-users") {
			      doSaveListToMain(name);
			      list.selectItem(addedItem);
			   }
			   else {
			      doSaveList(name);
			   }
   
   } catch(e) {
      debug(e, "doAddItem()");
   }    
}

//************************************************/
// edit selected item
//************************************************/

function doEditSelectedItem(name, display) {
   try {
			   var list = document.getElementById(name);
			
			   for(i = 0; i < list.selectedCount; i++) {
			      var item = list.getSelectedItem(i);
			      var value = window.prompt(autoformBundle.GetStringFromName("editbefore") + " " + display + " " + autoformBundle.GetStringFromName("editafter"), item.label);
			
			      if(display == 'regexp') {
			         try
			         {
			            myregexp = new RegExp(value);
			         }
			         catch(e) {
			            alert(autoformBundle.GetStringFromName("noregexp"));
			            value = "";
			         }
			      }
			
			      if(value != "" && value != null) {
			         item.label = value;
			      }
			   }
			
			   if(name == "autoform-users") {
			      doSaveListToMain(name);
			   }
			   else {
			      doSaveList(name);
			   }

   } catch(e) {
      debug(e, "doEditSelectedItem()");
   }    
}

//************************************************/
// save list to virtual user preference
//************************************************/

function doSaveList(name) {
   try {
			   var listPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("autoform." + this.autouser + "." + name + ".");
			   listPrefs.deleteBranch("");
			
			   var list = document.getElementById(name);
			
			   for(i = 0; i < list.getRowCount(); i++) {
			      listPrefs.setCharPref(list.getItemAtIndex(i).value, doHexEncode(list.getItemAtIndex(i).label));
			   }
   
   } catch(e) {
      debug(e, "doSaveList()");
   }   
}

//************************************************/
// save list to main preference
//************************************************/

function doSaveListToMain(name) {
	  try {
			   var listPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("autoform." + name + ".");
			   listPrefs.deleteBranch("");
			
			   var list = document.getElementById(name);
			
			   for(i = 0; i < list.getRowCount(); i++) {
			      listPrefs.setCharPref(list.getItemAtIndex(i).value, doHexEncode(list.getItemAtIndex(i).label));
			   }
			   
   } catch(e) {
      debug(e, "doSaveListToMain()");
   } 
}

//************************************************/
// check regular expression agains list
//************************************************/

function doCheckRegexp(url, list, status) {
   try {
			   var theUrl = document.getElementById(url);
			   var theList = document.getElementById(list);
			   var theStatus = document.getElementById(status);
			
			   for(i = 0; i < theList.getRowCount(); i++) {
			      if(theUrl.value.match(theList.getItemAtIndex(i).label)) {
			         theStatus.value = autoformBundle.GetStringFromName("urlmatch");
			         return;
			      }
			   }
			   
			   theStatus.value = autoformBundle.GetStringFromName("urlnomatch");
			   
   } catch(e) {
      debug(e, "doCheckRegexp()");
   } 
}

//************************************************/
// change user
//************************************************/

function doChangeUser() {
	  try {
			   autoPrefs.setCharPref("autouser", document.getElementById("autoform-users").selectedItem.value);
   			doLoadPreferences();
   			
   } catch(e) {
      debug(e, "doChangeUser()");
   }    			
}

   
//************************************************/
// Encodes the String to Hex values
//************************************************/
   
function doHexEncode(s) {
	  try {
			   var r = "0x";
			   var hexes = new Array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f");
			   for(var i = 0; i < s.length; i++) {
			      r += hexes[(s.charCodeAt(i) >> 12) & 0xf] + hexes[(s.charCodeAt(i) >> 8) & 0xf] + hexes[(s.charCodeAt(i) >> 4) & 0xf] + hexes[s.charCodeAt(i) & 0xf];
			   }
			   return r;

   } catch(e) {
      debug(e, "doHexEncode)");
   } 
}
   
//************************************************/
// Decodes the String to Hex values
//************************************************/
   
function _to_int(x1) {
	  try {
      return x1 >= 48 && x1 < 58 ? x1 - 48 : x1 - 97 + 10;
   } catch(e) {
      debug(e, "_to_int()");
   } 
}
   
function doHexDecode(s) {
	  try {
			   var r = "";
			   for(var i = 2; i < s.length; i += 4) {
			      x1 = _to_int(s.charCodeAt(i));
			      x2 = _to_int(s.charCodeAt(i + 1));
			      x3 = _to_int(s.charCodeAt(i + 2));
			      x4 = _to_int(s.charCodeAt(i + 3));
			      r += String.fromCharCode(((x1 << 12) & 0xF000) |((x2 << 8) & 0x0F00) |((x3 << 4) & 0x00F0) |(x4 & 0x000F));
			   }
			   return r;
			   
   } catch(e) {
      debug(e, "doHexDecode()");
   } 
}

//************************************************/
// Edit stored values
//************************************************/

function doEditEntry() {
	  try {
			   var tree = document.getElementById("autoform-prefs-tree");
			
			   var start = new Object();
			   var end = new Object();
			   var numRanges = tree.view.selection.getRangeCount();
						var showCannotEdit = false;
						
			   for (var t=0; t<numRanges; t++) {
			      tree.view.selection.getRangeAt(t,start,end);
			      
			      for (var v=start.value; v<=end.value; v++ ){
		      			var item = tree.view.getItemAtIndex(v);
			      			
		      			if (item.getAttribute("container") == false) {
							//edit item
							var prefRef = item.getAttribute("id");
							var itemDesc = document.getElementById("DESC" + prefRef);
							var itemValue = document.getElementById("VALUE" + prefRef);
							var itemType = document.getElementById("TYPE" + prefRef);
							
							var type = itemType.getAttribute("label");
							var value = itemValue.getAttribute("label");
							var elementId = itemValue.getAttribute("elementId");
							var desc = itemDesc.getAttribute("label");
							var branch = item.getAttribute("branch");
								
							if (type == 'text') {
								var newValue = prompt(autoformBundle.GetStringFromName("editField") + " " + desc + ':', value);
								itemValue.setAttribute("label", newValue);
								itemValue.setAttribute("value", doHexDecode(newValue));
								var branch = prefService.getBranch("autoform." + this.autouser + ".formdata." + branch);
								branch.setCharPref(elementId, doHexEncode(newValue));
	   						} else {
  								showCannotEdit = true;
												}
		      			}
			      }
			            
			   }
			   
			   if (showCannotEdit) {
			      alert(autoformBundle.GetStringFromName("onlyField"));
			   }
   
   } catch(e) {
      debug(e, "doEditEntry()");
   }    
}

//************************************************/
// Export 
//************************************************/

function doExport() {
			try {
				  // Choose File
						var nsIFilePicker = Components.interfaces.nsIFilePicker;
						var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
						fp.init(window, "Please enter a filename for the export", nsIFilePicker.modeSave);
						fp.appendFilter("Autoform Preference File","*.xml");
						
						var res=fp.show();
						if (res == nsIFilePicker.returnOK) {
							
							  // Open File
									var thefile = fp.file;
									var list = document.getElementById("autoform-users");
									var fileStream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
									fileStream.init(thefile, 0x02 | 0x08 | 0x20, 00640, 0);
					
								 // Write Header
									writeToFile(fileStream, "<?xml version=\"1.0\"?>");
									writeToFile(fileStream, "<autoform-preferences>");

									// Loop all selected users
									for(i = 0; i < list.selectedCount; i++) {
										
												// get users root branch
												var item = list.getSelectedItem(i);
												var dataBranch = prefService.getBranch("autoform." + item.value);
			   						var prefArray = dataBranch.getChildList("", prefCount);
			   						
			   						// initialize per user element
												writeToFile(fileStream, "\t<user-preferences name=\"" + encodeString(item.label) + "\">");
												writeToFile(fileStream, "\t\t<settings>");

						   			for(var p=0; p<prefCount.value; p++) {
						   						// get preference
						   						var pref = prefArray[p];
						   						var hasdata = pref.indexOf("formdata");
						   						if (hasdata == -1) {
						   									// user settings
						   									var thevalue = getUnknownPrefFromBranch("autoform." + item.value, pref);
						   									var thetype = getPrefTypeFromBranch("autoform." + item.value, pref);
						   									if (thetype == "char" && thevalue.substring(0,2) == "0x") {
						   										var encoded = "true";
						   										thevalue = doHexDecode(thevalue);
						   									} else {
						   										var encoded = "false";
						   									}
																		writeToFile(fileStream, "\t\t\t<setting name=\"" + encodeString(pref.substring(1, pref.length)) + "\" type=\"" + thetype + "\" encoded=\"" + encoded + "\">" + encodeString(thevalue) + "</setting>");
						   						}
						   			}
												writeToFile(fileStream, "\t\t</settings>");
						   			writeToFile(fileStream, "\t\t<formdata>");
						   			
						   			var ask = "true";
												
						   			for(var p=0; p<prefCount.value; p++) {
						   				
						   						// get preference
						   						var pref = prefArray[p];
						   						var hasdata = pref.indexOf("formdata");
						   						
						   						if (hasdata != -1) {
						   									// it is form data
						   									// get context (0 = host, 1 = path, 2 = querystring, 3 = formname, 4 = element
						   						   var context = new tokenizer(pref.substring(hasdata + 9, pref.length), ".");
						   						   // get element (0 = element name, 1 = element type)
						   						   var elm = new tokenizer(doHexDecode(context[4]), "@@@");
						   						   // get value
						   						   var theValue = "";
						   						   try {
						   						      theValue = doHexDecode(autoPrefs.getCharPref(item.value + pref));
						   						   } catch (e) {
						   						   	  //ignore, can be empty;
						   						   }
						   						   if (elm[1] == "select-one" || elm[1] == "select-multiple") {
				  			         				// make human selects readable
																		    var descTokens = new tokenizer(theValue, "|@|");
																			   var desc = "";
																			
																			   for (t=0; t<descTokens.length; t++) {
																			      desc += descTokens[t];
																			      if (t != descTokens.length -1) {
																			   	     desc += ",";
																			      }
																		   }
																			
									              theValue = desc;
									              
					           } else {
					           	  // handle passwords
																			if (elm[1] == 'password' && theValue != "") {
																							theValue = desDecode(theValue, ask);
																						 if (theValue == null) {
						   						      	    ask = "false";
																								  theValue="******";
						   						        }
																			}
					           }
				           
				           			// and write finally the data
						   						   writeToFile(fileStream, "\t\t\t<data host=\"" + encodeString(doHexDecode(context[0])) + "\" path=\"" + encodeString(doHexDecode(context[1]))+ "\" querystring=\"" + encodeString(doHexDecode(context[2]))+ "\" form=\"" + encodeString(doHexDecode(context[3]))+ "\" element=\"" + encodeString(elm[0]) + "\" type=\"" + encodeString(elm[1]) + "\">" + encodeString(theValue) + "</data>");
						   						   
						   						}
						   			}
												writeToFile(fileStream, "\t\t</formdata>");

												writeToFile(fileStream, "\t</user-preferences>");
									}
			
									writeToFile(fileStream, "</autoform-preferences>");
									fileStream.close();
						}

			} catch(e) {
						debug(e, "doExport()");
			}    
}

//encode xml string
function encodeString(s) {
					str = new String(s);
	   str = str.replace(/&/g,'&amp;');
	   str = str.replace(/>/g,'&gt;');
	   str = str.replace(/</g,'&lt;');
	   str = str.replace(/\"/g,'&quot;');
	   str = str.replace(/\'/g,'&apos;');
	   return str;
}

// write a line to the file
function writeToFile(file, text) {
	  text += "\n";
			file.write(text, text.length);
}

//************************************************/
// Import 
//************************************************/

function doImport() {
			try {
				  // get import file
						var nsIFilePicker = Components.interfaces.nsIFilePicker;
						var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
						fp.init(window, "Please choose the preferences to import", nsIFilePicker.modeOpen);
						fp.appendFilter("Autoform Preference File","*.xml");
						
						var res=fp.show();
						
						if (res==nsIFilePicker.returnOK) {

									// read file into string
									var thefile=fp.file;
									var inputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
									inputStream.init(thefile, 0x01, 0444, null);
									var scriptableIO = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
									scriptableIO.init(inputStream);
									var fileString = scriptableIO.read(thefile.fileSize-1);
									scriptableIO.close();
									inputStream.close();
									
									// make a dom of the xml file
									var domParser = Components.classes["@mozilla.org/xmlextras/domparser;1"].createInstance(Components.interfaces.nsIDOMParser);
									try {
												var domDocument = domParser.parseFromString(fileString, "text/xml");
												var usersBranch = prefService.getBranch("autoform.autoform-users");
												var userList = usersBranch.getChildList("", prefCount);
												var userNodeList = domDocument.getElementsByTagName("user-preferences");
												
												// loop each exported user
												for (var u=0; u<userNodeList.length; u++) {
															var userNode = userNodeList.item(u);
															var userName = userNode.attributes.getNamedItem("name").value;
															
															found = false;
															for (var su=0; su<userList;su++) {
																  if (userList[su] == userName) {
																  	  found = true;
																  }
															}
															
															if (found && 	confirm("User " + userName + " already exists! Do you really want to import these settings!")) {		
																
																  var importPrefs = prefService.getBranch("autoform." + this.autouser + ".");
																											
																		//import settings
																		var settingsElm = userNode.getElementsByTagName("settings").item(0);
																		var settingList = settingsElm.getElementsByTagName("setting");
																		
																		for (var s=0; s<settingList.length; s++) {
																					var setting = settingList.item(s);
																					var name = setting.attributes.getNamedItem("name").value;
																					var type = setting.attributes.getNamedItem("type").value;
																					var encoded = setting.attributes.getNamedItem("encoded").value;
																		   var value = setting.firstChild.nodeValue;
																		   
																		   if (encoded == "true") {
																		   	  value = doHexEncode(value);
																		   }
																		   
																		   if (type == 'bool') {
																		      importPrefs.setBoolPref(name, value);
																		   } else if (type == 'char') {
																		      importPrefs.setCharPref(name, value);
																		   }
																		}
																		
																		//import data
																		var datas = userNode.getElementsByTagName("data");
																		for (var d=0; d<datas.length; d++) {
																					var data = datas.item(d);
																					var host = data.attributes.getNamedItem("host").value;
																					var path = data.attributes.getNamedItem("path").value;
																					var querystring = data.attributes.getNamedItem("querystring").value;
																					var form = data.attributes.getNamedItem("form").value;
																					var element = data.attributes.getNamedItem("element").value;
																					var type = data.attributes.getNamedItem("type").value;
																					
																					var importPrefs = prefService.getBranch("autoform." + this.autouser + ".formdata." + doHexEncode(host) + "." + doHexEncode(path) + "." + doHexEncode(querystring) + "." + doHexEncode(form) + "." + doHexEncode(element + "@@@" + type));
																					
																		   var value = data.firstChild.nodeValue;
																		   
																		   importPrefs.set
																					
																		}
															}
												}
												
												doLoadPreferences();
												
									} catch (e) {
													alert("Autoform cannot read preference file: " + e);
									}
						}

			} catch(e) {
						debug(e, "doImport()");
			}      
}

function decodeString(s) {
				str = new String(s);
	   str = str.replace(/&amp;/,'&');
	   str = str.replace(/&gt;/,'>');
	   str = str.replace(/&lt;/,'<');
	   str = str.replace(/&quot;/,'"');
	   str = str.replace(/&apos;/,"'");
	   return str;
}
   //************************************************/
   // Decode encrypted text
   //************************************************/
   
   function desDecode(text, ask) {
   			try {
			      if(masterpassword == null && ask == "true") {
			         getMasterPassword();
			      }

			      if(masterpassword == null && ask == "true") {
			         alert(autoformBundle.GetStringFromName("noMasterPassword"));
			         return null;
		      } else if(masterpassword == null && ask == "false") {
			     			return null;
			     } else {
						     return des(masterpassword, doHexDecode(text), 0);
			     }
			      
   		 } catch(e) {
   		    debug(e, "desDecode()");
   		 }   			
   }
   
   //************************************************/
   // Encode encrypted text
   //************************************************/
   
   function desEncode(text) {
						try {
		      if(masterpassword == null) {
		         getMasterPassword();
		      }

		      if(masterpassword == null) {
		         alert(autoformBundle.GetStringFromName("noMasterPassword"));
			        return null;
		      } else {		      
   		      return doHexEncode(des(masterpassword, text, 1));
		      }

   		 } catch(e) {
   		    debug(e, "desEncode()");
   		 }   			
   }
   
   //************************************************/
   // set master password for encryption
   //************************************************/
   
   function getMasterPassword() {
   	  try {
			      // get a reference to the prompt service component.
      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
      var pwdObj = new Object();
      var checkObj = new Object();

      if(promptService.promptPassword(window, autoformBundle.GetStringFromName("masterpassword"), autoformBundle.GetStringFromName("entermasterpassword"), pwdObj, null, checkObj)) {
			         masterpassword = pwdObj.value;
			   }

   		 } catch(e) {
   		    debug(e, "getMasterPassword()");
   		 }   			
   }
   
//************************************************/
// string tokenizer
//************************************************/

function tokenizer(stringValue, delimiter) {
	  try {
			   begin = 0;
			   end = stringValue.indexOf(delimiter);
			   tokenCount = 0;

			   while(stringValue.indexOf(delimiter, begin) > 0) {
			      this[tokenCount] = stringValue.substring(begin, end);
			      tokenCount += 1;
			      begin = end + delimiter.length;
			      end = stringValue.indexOf(delimiter, begin);
			      this.length = tokenCount;
			   }
			
			   this[tokenCount] = stringValue.substring(begin, stringValue.length);
   } catch(e) {
      debug(e, "tokenizer()");
   }    
}

//************************************************/
// Debug messages
//************************************************/

function debug(text, method) {
   		var msg = "-------------------------------------\n" +
   												"An error within autoform has occured!\n" + 
   												"-------------------------------------\n" +
   												"Error source: optioncontent.js\n" +
   												"Error method: " + method +"\n" +
   												"Error message: " + text + "\n" +
   												"-------------------------------------\n" +
															"Please send this details to improve the\n" +
															"quality of this extension to:\n" +
															"michael.kessler@team.bluewin.com\n" +
   												"-------------------------------------\n";
						if (document.getElementById("autoform-erroralert").checked) {						
			   			alert(msg);
						} else {
							  dumnp(msg);
						}
}